# Blunt

[![Gem Version](https://badge.fury.io/rb/blunt.svg)](https://badge.fury.io/rb/blunt)

**Blunt** is a token-based authentication library for Hanami. Its goal is to easily implement a single authentication scheme across multiple interfaces, i.e. both web/multi-page and mobile/SPA/API/async.

## Installation

Come on now:

```ruby
gem 'blunt'
```
```bash
bundle install
```

Or:

```bash
gem install blunt
```

## Setup

### Environment

- `ENV['BLUNT_SECRET']` (String)

### Models

- `User` entity (+ `UserRepository`)
    + `email`
    + `password_digest`
- `RefreshToken` entity (+ `RefreshTokenRepository`)
    + `user_id`
    + `valid` (Boolean)
    + timestamps

### Token Handling

Blunt offers several strategies for token storage. Just include the appropriate module, and Blunt will handle tokens on the back-end - they never need to touch your code!

```ruby
Hanami::Controller.configure do
  prepare do
    include Blunt::Controller::__STRATEGY__
  end
end
```

#### Cookies

`Blunt::Controller::Web`

The ultimate hands-off approach. No setup is required on the front-end. Ideal for multi-page apps.

TODO: incorporate asynchronous JSON responses

#### HTTP Headers

`Blunt::Controller::API`

TODO: develop this

Pass the token from the client in a request header: `'HTTP_AUTHORIZATION' => 'Bearer <TOKEN>'`.

#### Custom Strategy

Define a few methods to govern the access token lifecycle as it gets passed to and from the client. Blunt will use these 3 methods to handle the token. The example below shows a simple implementation for synchronous requests using cookies, but a custom strategy will likely be more complex.

```ruby
module MyCustomStrategy
  include Blunt::Controller

  private
  def set_access_token(token)
    # Login or renew
    cookies[:access_token] = token
  end

  def get_access_token
    # Authenticate
    cookies[:access_token]
  end

  def unset_access_token
    # Logout
    cookies[:access_token] = nil
  end
end
```

## Usage

### Signup

```ruby
# your signup interactor
class MySignupClass < Thalamus::Interactor
  validations do
    # password length, complexity, etc
    # ...
  end
  def call
    @user = Blunt.signup(params)
  end
  expose :user
end
# inside your signup controller
if user = MySignupClass.call(params).user
  login!(user)
  # ...
else
  # ...
end
```

`Blunt.signup` compares a `:password` and `:password_confirmation` field. If they match, it creates a new `User` and saves it to the database with a `:password_digest`. If they don't match, it returns nil.

If you don't use the interactor pattern, You can also use the shortcut `#signup!` directly from your controller. However, you may wish to validate user data and password complexity yourself.

```ruby
# inside your signup controller
if signup!(params)
  # Blunt calls #login! under the hood
  # ...
else
  # ...
end
```

### Login

```ruby
# inside your login controller
token = login!(params)
# ...
```

`#login!` finds a user with `:email` and checks that `:password` is correct. Returns an access token if successful, otherwise nil.

### Request Authentication

All strategies provide `#current_user` (`User` entity or nil) and `#authenticated?` (Boolean) instance methods to the controller. The token will not be checked until one of these methods is called.

### Logout

```ruby
# inside your logout controller
logout!
# ...
```

That's it!
